@@ -23,6 +23,8 @@ use Apache::TestConfig ();
 
 use vars qw(@ISA @EXPORT %EXPORT_TAGS $VERSION %SubTests @SkipReasons);
 
+$VERSION = '1.25';
+
 my @need = qw(need_lwp need_http11 need_cgi need_access need_auth
               need_module need_apache need_min_apache_version
               need_apache_version need_perl need_min_perl_version
@@ -40,8 +42,6 @@ my @test_more_exports = grep { ! /^(ok|skip|plan)$/ } @EXPORT;
 
 %EXPORT_TAGS = (withtestmore => \@test_more_exports);
 
-$VERSION = '1.16';
-
 %SubTests = ();
 @SkipReasons = ();
 
@@ -132,7 +132,6 @@ EOE
 #so Perl's Test.pm can be run inside mod_perl
 sub test_pm_refresh {
     if (@testmore) {
-        
         Test::Builder->reset;
 
         Test::Builder->output(\*STDOUT);
@@ -156,12 +155,12 @@ sub test_pm_refresh {
 sub init_test_pm {
     my $r = shift;
 
-    # needed to load Apache::RequestRec::TIEHANDLE
-    eval {require Apache::RequestIO};
-    if (defined &Apache::RequestRec::TIEHANDLE) {
+    # needed to load Apache2::RequestRec::TIEHANDLE
+    eval {require Apache2::RequestIO};
+    if (defined &Apache2::RequestRec::TIEHANDLE) {
         untie *STDOUT;
         tie *STDOUT, $r;
-        require Apache::RequestRec; # $r->pool
+        require Apache2::RequestRec; # $r->pool
         require APR::Pool;
         $r->pool->cleanup_register(sub { untie *STDOUT });
     }
@@ -278,10 +277,9 @@ sub need_module {
     for (@modules) {
         if (/^[a-z0-9_.]+$/) {
             my $mod = $_;
-            unless ($mod =~ /\.c$/) {
-                $mod = 'mod_' . $mod unless $mod =~ /^mod_/;
-                $mod .= '.c'
-            }
+            $mod .= '.c' unless $mod =~ /\.c$/;
+            next if $cfg->{modules}->{$mod};
+            $mod = 'mod_' . $mod unless $mod =~ /^mod_/;
             next if $cfg->{modules}->{$mod};
             if (exists $cfg->{cmodules_disabled}->{$mod}) {
                 push @reasons, $cfg->{cmodules_disabled}->{$mod};
@@ -604,7 +602,7 @@ one C or Perl module from the list cannot be found).
 the tests will be skipped if the function returns a false value. For
 example:
 
-    plan tests => 5, \&need_lwp;
+    plan tests => 5, need_lwp;
 
 the test will be skipped if LWP is not available
 
@@ -654,13 +652,13 @@ server versions.
 
 =item need_http11
 
-  plan tests => 5, &need_http11;
+  plan tests => 5, need_http11;
 
 Require HTTP/1.1 support.
 
 =item need_ssl
 
-  plan tests => 5, &need_ssl;
+  plan tests => 5, need_ssl;
 
 Require SSL support.
 
@@ -668,13 +666,13 @@ Not exported by default.
 
 =item need_lwp
 
-  plan tests => 5, &need_lwp;
+  plan tests => 5, need_lwp;
 
 Require LWP support.
 
 =item need_cgi
 
-  plan tests => 5, &need_cgi;
+  plan tests => 5, need_cgi;
 
 Requires mod_cgi or mod_cgid to be installed.
 
@@ -767,18 +765,6 @@ arguments or a reference to a list.
 In case of C modules, depending on how the module name was passed it
 may pass through the following completions:
 
-=item need_min_module_version
-
-Used to require a minimum version of a module
-
-For example:
-
-  plan tests => 5, need_min_module_version(CGI => 2.81);
-
-requires C<CGI.pm> version 2.81 or higher.
-
-Currently works only for perl modules.
-
 =over
 
 =item 1 need_module 'proxy_http.c'
@@ -798,6 +784,18 @@ lookup, turning it into I<'mod_cgi.c'>.
 
 =back
 
+=item need_min_module_version
+
+Used to require a minimum version of a module
+
+For example:
+
+  plan tests => 5, need_min_module_version(CGI => 2.81);
+
+requires C<CGI.pm> version 2.81 or higher.
+
+Currently works only for perl modules.
+
 =item need
 
   plan tests => 5,
